import random
import time
from typing import Union, List

from nonebot.adapters.onebot.v11 import Bot, Message, GroupMessageEvent, PrivateMessageEvent
from nonebot.matcher import Matcher
from nonebot.params import CommandArg

from .Ess import ess
from .Card import card

class Le:
    def __init__(self) -> None:
        pass

    async def lulu(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent]) -> None:
        if isinstance(event, GroupMessageEvent) and event.group_id in ess.config['modules']['Le']:
            return None
        
        effect = random.randint(0, 2)
        if effect == 0:
            await matcher.finish(f'露露的幸运蛋结果：\n获得{random.randint(1, 6)*20}Stars')
        elif effect == 1:
            await matcher.finish('露露的幸运蛋结果：\n抽取五张卡')
        else:
            await matcher.finish('露露的幸运蛋结果：\n恢复所有HP，获得永久ATK/DEF/EVD+1')

    async def nanako(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent]) -> None:
        if isinstance(event, GroupMessageEvent) and event.group_id in ess.config['modules']['Le']:
            return None

        list1 = []
        for i in range(0, 2):
            a = random.randint(0, 7)
            list1.append(a)
        list1.sort()
        list1.append(7)

        points = []
        for i in range(len(list1)):
            b = list1[i] if i == 0 else list1[i] - list1[i-1]
            points.append(b)
        await matcher.finish(f'浮游炮展开结果：\nATK+{points[0]} DEF+{points[1]} EVD+{points[2]}')
    
    async def nico(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        if isinstance(event, GroupMessageEvent) and event.group_id in ess.config['modules']['Le']:
            return None

        text = arg.extract_plain_text()
        num = min(int(text), 8) if (text := text[:1]).isdigit() and int(text) else 4
    
        ordinary_list = []
        for tuple in card.cardHyper:
            if tuple[2] == 'ordinary':
                ordinary_list.append(tuple)
    
        result = []
        for i in range(0, num):
            result.append(random.choice(ordinary_list)[0])
        await matcher.finish(f'奇迹漫步结果：\n{", ".join(result)}')

    async def divination(self, bot: Bot, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        if isinstance(event, GroupMessageEvent) and event.group_id in ess.config['modules']['Le']:
            return None
        
        arg = arg.extract_plain_text()
        now = time.localtime()
        seed = round(abs(hash("Polaris_Light" + str(now.tm_yday)+ str(now.tm_year) + "XYZ") / 3.0 + hash("QWERTY" + arg + str(event.user_id) + "0*8&6" + str(now.tm_mday) + "kjhg") / 3.0))
        random.seed(seed)

        divination_list: List[str] = ['大吉', '吉', '中吉', '小吉', '半吉', '末吉', '末小吉', '凶', '小凶', '半凶', '末凶', '大凶']
        result = random.choice(divination_list)

        if '女装' in arg:
            result = '必须大吉！建议马上开始'

        for name in bot.config.nickname:
            if bot.config.nickname in arg:
                result = '你好像有那个大病'

        user_name = event.sender.card if event.sender.card != '' else event.sender.nickname
        now = time.localtime()
        await matcher.finish(f'今天是{now.tm_year}年{now.tm_mon}月{now.tm_mday}日\n{user_name}所求事项：【{arg}】\n\n结果：【{result}】')


le = Le()


from nonebot.plugin import PluginMetadata

from .Config import Config

__plugin_meta__ = PluginMetadata(
    name='100orangejuice',
    description='100orangejuice',
    usage= r'See Sorabot Docs.',
    type="application",
    supported_adapters={"~onebot.v11"},
    homepage="https://github.com/FDCraft/nonebot-plugin-orangejuice",
    config=Config,
    extra={
        'author': 'Polaris_Light',
        'version': '0.4.0',
        'priority': 10
    }
)

import re

from nonebot import on_command, on_regex
from nonebot.matcher import Matcher

from .Ess import ess

from .Card import card
from .Deck import deck
from .Emote import emote
from .Le import le
from .Stats import stats

@on_command('#help', aliases={'#帮助'}, priority=10, block=True).handle()
async def help(matcher: Matcher):
    help_msg = '''100OrangeJuice
    #ess help 基础管理模块帮助
    #card help 查卡帮助
    #icon help 查图标帮助
    #stat help 统计帮助
    #deck=[code] 卡组查看
    #lulu 幸运蛋
    #mw [num] 奇迹漫步
    #7 浮游炮
    #zb [something] 占卜'''
    await matcher.finish(help_msg)

on_command(
    '#ess',
    aliases={'#Ess', '#Essential', '#essentilasx'},
    priority=10,
    block=True,
    handlers=[ess.ess]
)

on_command(
    '#card',
    priority=10,
    block=True,
    handlers=[card.card]
)

on_command(
    '#icon',
    priority=10,
    block=True,
    handlers=[card.icon]
)

on_command(
    '#stat',
    aliases={'#stats', '#sd'},
    priority=10,
    block=True,
    handlers=[stats.stats]
)

on_command(
    '#deck=',
    priority=10,
    block=True,
    handlers=[deck.deck]
)

on_command(
    '#lulu',
    aliases={'#幸运蛋'},
    priority=10,
    block=True,
    handlers=[le.lulu]
)

on_command(
    '#mw',
    aliases={'#奇迹漫步'},
    priority=10,
    block=True,
    handlers=[le.nico]
)

on_command(
    '#7',
    aliases={'#浮游炮'},
    priority=10,
    block=True,
    handlers=[le.nanako]
)

on_command(
    '#zb',
    aliases={'#占卜'},
    priority=10,
    block=True,
    handlers=[le.divination]
)


on_regex(
    r"^\:[a-z0-9_]+\:$",
    flags=re.IGNORECASE,
    handlers=[emote.emote])





import json
import os
from typing import Dict, Union

from nonebot.adapters.onebot.v11 import GroupMessageEvent, PrivateMessageEvent, Message, MessageSegment
from nonebot.matcher import Matcher
from nonebot.params import CommandArg

from .Config import plugin_config
from .Ess import ess

steam_id_file_path: str = os.path.join(plugin_config.oj_data_path, 'steam_id.json')

class Stats:
    def __init__(self) -> None:
        if not os.path.exists(steam_id_file_path):
            if not os.path.exists(plugin_config.oj_data_path):
                os.makedirs(plugin_config.oj_data_path)
                                
            with open(steam_id_file_path, 'w+') as f:
                """
                steam_id.json
                {
                    "user_id": "steam_id"
                }
                """

                init_json = {}
                f.write(json.dumps(init_json, indent=4))

    async def help(self, matcher: Matcher) -> None:
        help_msg: str = '''橙汁个人统计图片生成
    #stat [steam64id] [limit]
    用于直接生成。limit为行数，可以不填，默认为5。
    使用非初始样式时，不会生成PVP各角色出场次数和胜场的行。
    #stat bind [steam64id]
    用于将当前账号绑定到对应steam账户。重复使用会更新绑定。
    #stat unbind
    删除自己的绑定。
    #stat me [limit]
    绑定steam64id后，使用本命令来快速生成自己的资料。'''
        await matcher.finish(help_msg)

    async def bind(self, uid: str, steam64id: str, matcher: Matcher) -> None:
        try:
            with open(steam_id_file_path, 'r', encoding='utf-8') as f:
                data: Dict[str, str] = json.load(f)
            data[uid] = steam64id
            json_data = json.dumps(data, indent=4)
            with open(steam_id_file_path, 'w', encoding='utf-8') as f:
                f.write(json_data)
            await matcher.send('绑定成功~')
        except:
            await matcher.finish('诶鸭出错啦~绑定失败~')

    async def unbind(self, uid: str, matcher: Matcher) -> None:
        try:
            with open(steam_id_file_path, 'r', encoding='utf-8') as f:
                data: Dict[str, str] = json.load(f)
            flag: bool = bool(data.pop(uid, False))
            if flag:
                with open(steam_id_file_path, 'w', encoding='utf-8') as f:
                    json_data = json.dumps(data, indent=4)
                    f.write(json_data)
                await matcher.send('解除绑定成功~')
            else:
                await matcher.finish('你还没有绑定过steam哦~')
        except:
            await matcher.finish('诶鸭出错啦~解绑失败~')

    async def me(self, uid: str, limit: str, matcher: Matcher) -> None:
        try:
            with open(steam_id_file_path, 'r', encoding='utf-8') as f:
                data: Dict[str, str] = json.load(f)
                steam64id: Union[str, None] = data.get(uid, None)
            if steam64id == None:
                await matcher.send('你还没有绑定过steam哦~')
            else:
                img = f'https://interface.100oj.com/stat/render.php?steamid={steam64id}&limit={limit}'
                await matcher.send(MessageSegment.image(img))
        except:
            await matcher.finish('诶鸭出错啦~')

    async def get_img(self, steam64id: str, limit: str, matcher: Matcher) -> None:
        try:
            img = f'https://interface.100oj.com/stat/render.php?steamid={steam64id}&limit={limit}'
            await matcher.send(MessageSegment.image(img))
        except:
            await matcher.finish('诶鸭出错啦~')

    async def stats(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        if isinstance(event, GroupMessageEvent) and event.group_id in ess.config['modules']['Stats']:
            return None

        args = arg.extract_plain_text().split(' ')

        if args == [''] or args is None or args[0] == 'help':
            await self.help(matcher)

        if args[0] == 'bind':
            uid = str(event.user_id)

            if len(args) == 2:
                steam64id: str = args[1]
                if steam64id.isdigit() and len(steam64id) == 17:
                    await self.bind(uid, steam64id, matcher)
                else:
                    await self.help(matcher)
            else:
                await self.help(matcher)

        elif args[0] == 'unbind':
            uid = str(event.user_id)
            await self.unbind(uid, matcher)

        elif args[0] == 'me':
            uid = str(event.user_id)
            if len(args) == 2:
                if args[1].isdigit():
                    limit: str = str(min(int(args[1]), 10))
                    await self.me(uid, limit, matcher)
                else:
                    await self.help(matcher)
            else:
                limit: str = '5'
                await self.me(uid, limit, matcher)
        
        elif args[0].isdigit() and len(args[0]) == 17:
            steam64id = args[0]
            if len(args) == 2:
                if args[1].isdigit():
                    limit: str = str(min(int(args[1]), 10))
                    await self.get_img(steam64id, limit, matcher)
                else:
                    await self.help(matcher)
            else:
                limit: str = '5'
                await self.get_img(steam64id, limit, matcher)
        
        else:
            await self.help(matcher)


stats = Stats()
            


from typing import Tuple, Union

import aiomysql
import asyncio

from nonebot.adapters.onebot.v11 import Message, MessageSegment, GroupMessageEvent, PrivateMessageEvent 
from nonebot.matcher import Matcher
from nonebot.params import CommandArg

from .Ess import ess

BaseClass = Tuple[Tuple[str]]

tables = ['cardDeck', 'cardHyper', 'cardOther', 'unitCharacter', 'unitNPC']
groups = {'原版': 'ordinary', '合作': 'coop', '赏金': 'bounty', '其他': 'other'}

class Card:
    def __init__(self) -> None:
        pass
    async def get_data(self):
        self.db = await aiomysql.connect(
            host='47.242.108.196',
            port=3306,
            user='guest',
            password='Sumika!System2',
            db='data_oj'
        )

        cursor = await self.db.cursor()

        for table in tables:
            await cursor.execute(f"SELECT name_, i18nkey, groupData FROM {table}")
            setattr(self, table, await cursor.fetchall())
        
        self.db.close()
        return self

    async def help(self, matcher: Matcher):
        help_msg: str = '''橙汁查卡
    #card [name] [group]
    查卡
    #icon [name] [group]
    查图标
    name为中文卡名
    group为 原版/合作/赏金/其他 或不填'''
        await matcher.send(help_msg)

    async def card(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        try:
            if isinstance(event, GroupMessageEvent) and event.group_id in ess.config['modules']['Card']:
                return None
            args = arg.extract_plain_text().split(' ')

            if args == [''] or args is None or args[0] == 'help':
                await self.help(matcher)
                return None

            name = args[0]

            if len(args) == 2 and args[1] in groups.keys():
                group = groups[args[1]]
            else:
                group = None
            
            id = ''
            for table in tables:
                table_attr: BaseClass = getattr(self, table)
                for tuple in table_attr:
                    if group != None:
                        if tuple[0] == name and tuple[2] == group:
                            id = tuple[1]
                            now_table = table
                            break                            
                    else:
                        if tuple[0] == name:
                            id = tuple[1]
                            now_table = table
                            if tuple[2] == 'ordinary':
                                break
                if id != '':
                    break
            
            if id == '':
                await matcher.send('没有这张卡啦~')
                return None

            if now_table.startswith('unit'):
                img = f'http://interface.100oj.com/interface/render/cardunit.php?key={id}'
                await matcher.send(MessageSegment.image(img))
                return None
            else:
                img = f'http://interface.100oj.com/interface/render/{now_table.lower()}.php?key={id}'
                await matcher.send(MessageSegment.image(img))
                return None                               

        except:
            await matcher.finish('诶鸭出错啦~')

    async def icon(self, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        try:
            if isinstance(event, GroupMessageEvent) and event.group_id in ess.config['modules']['Card']:
                return None
            args = arg.extract_plain_text().split(' ')

            if args == [''] or args is None or args[0] == 'help':
                await self.help(matcher)
                return None

            name = args[0]

            if len(args) == 2 and args[1] in groups.keys():
                group = groups[args[1]]
            else:
                group = None
            
            id = ''
            for table in tables:
                table_attr: BaseClass = getattr(self, table)
                for tuple in table_attr:
                    if group != None:
                        if tuple[0] == name and tuple[2] == group:
                            id = tuple[1]
                            break                            
                    else:
                        if tuple[0] == name:
                            id = tuple[1]
                            if tuple[2] == 'ordinary':
                                break
                if id != '':
                    break
            
            if id == '':
                await matcher.send('没有这张卡啦~')
                return None

            img = f'http://interface.100oj.com/interface/util/icon.php?key={id}&size=256&lossless=true'
            await matcher.send(MessageSegment.image(img))
            return None                     
        except:
            await matcher.finish('诶鸭出错啦~')

card = asyncio.run(Card().get_data())


import json
import os
from typing import Annotated, Dict, List, Literal, Union

from nonebot import logger
from nonebot.exception import ParserExit
from nonebot.matcher import Matcher
from nonebot.params import CommandArg
from nonebot.adapters.onebot.v11 import Bot, Message, GroupMessageEvent, PrivateMessageEvent

from .Config import plugin_config

ess_file_path: str = os.path.join(plugin_config.oj_data_path, 'essentialx.json')
init_json: Dict[str, Union[Dict[str, List[str]], int]] = {"modules":{"Card":[],"Deck":[],"Emote":[],"Le":[],"Stats":[]},"version":1}

class Args:
    def __init__(self, bot: Bot, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message) -> None:
        global init_json

        self.invaild: bool = False
        self.handle: str = None
        self.operate: str = None
        self.target_module: str = None
        self.target_uid: int = None
        self.target_gid: int = None
        self.send_uid: int = event.user_id
        self.send_gid: int = event.group_id if isinstance(event, GroupMessageEvent) else None
        if isinstance(self.send_gid, tuple): #IDK :(
            self.send_gid = self.send_gid[0]
        self.is_admin: bool = event.sender.role in ["admin", "owner"] if isinstance(event, GroupMessageEvent) else False
        self.is_superuser: bool = str(event.user_id) in bot.config.superusers

        list_args = arg.extract_plain_text().split(' ')

        while len(list_args) < 5:
            list_args.append('')
        logger.debug(f'参数解析结果：{list_args}')

        match list_args[0]:
            case 'module' | '-m':
                self.handle = 'module'

                match list_args[1]:
                    case 'list' | '-l' | 'ls':
                        self.operate = 'list'

                        self.invaild = True # ##ess module list [gid]
                        self.target_gid = int(list_args[2]) if list_args[2].isdigit() else self.send_gid

                    case 'enable' | '-on':
                        self.operate = 'enable'
                        logger.error('?')
                        if list_args[2] in list(init_json['modules'].keys()):
                            self.invaild = True # ##ess module enable [modulename] [gid]
                            self.target_module = list_args[2]
                            self.target_gid = int(list_args[3]) if list_args[3].isdigit() else self.send_gid


                    case 'disable' | '-off':
                        self.operate = 'disable'
                        logger.error('?')
                        if list_args[2] in list(init_json['modules'].keys()):
                            self.invaild = True # ##ess module disable [modulename] [gid]
                            self.target_module = list_args[2]
                            self.target_gid = int(list_args[3]) if list_args[3].isdigit() else self.send_gid
            case _:
                pass


class Ess:
    def __init__(self) -> None:
        if not os.path.exists(ess_file_path):
            if not os.path.exists(plugin_config.oj_data_path):
                os.makedirs(plugin_config.oj_data_path)
            self.init_json()  

        self.load_json()

    def init_json(self):
        with open(ess_file_path, 'w+') as f:
            '''
            essential.json
            {
                "modules": {
                    "Card": [], 
                    "Deck": [], 
                    "Emote": [], 
                    "Le": [],
                    "Stats": []
                },
                "version": 1
            }
            '''
            f.write(json.dumps(init_json, indent=4))

    def load_json(self):
        with open(ess_file_path, 'r', encoding='utf-8') as f:
            data: Dict[str, Union[Dict[str, List[str]], int]] = json.load(f)
            self.config = data

    def save_json(self):
        with open(ess_file_path, 'w', encoding='utf-8') as f:
            json_data = json.dumps(self.config, indent=4)
            f.write(json_data)
    

    async def help(self, matcher: Matcher) -> None:
        help_msg: str = '''Ess模块，需要是超级管理员/群管理/群主
    #ess module enable <modulename> [groupid]
    #ess module disable <modulename> [groupid]
    #ess module list [groupid]'''
        await matcher.finish(help_msg)

    async def module(self, matcher: Matcher, args: Args) -> None:

        if args.operate == 'enable':
            if args.target_gid in self.config['modules'][args.target_module]:
                self.config['modules'][args.target_module].remove(args.target_gid)
            self.save_json()
            await matcher.send(f'已在{args.target_gid}成功启用{args.target_module}模块！')

        elif args.operate == 'disable':
            if args.target_gid not in self.config['modules'][args.target_module]:
                self.config['modules'][args.target_module].append(args.target_gid)
            self.save_json()
            await matcher.finish(f'已在{args.target_gid}成功禁用{args.target_module}模块！')

        elif args.operate == 'list':
            enable_modules = []
            avaliable_modules = []
            for module in self.config['modules'].keys():
                if args.target_gid not in self.config['modules'][module]:
                    enable_modules.append(module)
                else:
                    avaliable_modules.append(module)
            msg1 = "\n- " + "\n- ".join(enable_modules) if enable_modules != [] else ""
            msg2 = "\n- " + "\n- ".join(avaliable_modules) if avaliable_modules != [] else ""
            await matcher.finish(f'群{args.target_gid}\n启用的模块: {msg1}\n可用模块: {msg2}')

        else:
            await self.help(matcher)

    async def ess(self, bot: Bot, matcher: Matcher, event: Union[GroupMessageEvent, PrivateMessageEvent], arg: Message = CommandArg()) -> None:
        args = Args(bot, event, arg)

        if not args.is_admin and not args.is_superuser:
            return None
        
        if not args.invaild:
            await self.help(matcher)
        
        if args.target_gid == None:
            await matcher.finish('呜，请指定群号！')

        if args.target_gid != args.send_gid and not args.is_superuser:
            await matcher.finish('诶鸭你没有权限这样做啦！')

        if hasattr(args, "handle"):
            await getattr(self, args.handle)(matcher, args)
            
        
ess = Ess()
